/*********************************************************************
*               SEGGER MICROCONTROLLER GmbH & Co KG                  *
*       Solutions for real time microcontroller applications         *
**********************************************************************
*                                                                    *
*       (c) 1995 - 2011  SEGGER Microcontroller GmbH & Co KG         *
*                                                                    *
*       www.segger.com     Support: support@segger.com               *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS * Real time operating system for microcontrollers      *
*                                                                    *
*                                                                    *
*       Please note:                                                 *
*                                                                    *
*       Knowledge of this file may under no circumstances            *
*       be used to write a similar product or a real-time            *
*       operating system for in-house use.                           *
*                                                                    *
*       Thank you for your fairness !                                *
*                                                                    *
**********************************************************************
*                                                                    *
*       OS version: 3.82r                                            *
*                                                                    *
**********************************************************************

----------------------------------------------------------------------
File    : BSP.c
Purpose : BSP for AVR32UC EVK1100 (Rev. C)
--------  END-OF-HEADER  ---------------------------------------------
*/

#include "BSP.h"

/*********************************************************************
*
*       Defines
*
**********************************************************************
*/

/****** Addresses of sfrs and peripheral identifiers ****************/
#define AVR32_GPIO_BASE_ADDR         (0xFFFF1000)

#define AVR32_GPIO_PIOB_BASE_ADDR    (AVR32_GPIO_BASE_ADDR + 0x100)

#define PIO_GPER_OFFS                (0x00) // GPIO Enable Register
#define PIO_ODR_OFFS                 (0x40) // Output Driver Enable Register 
#define PIO_OVR_OFFS                 (0x50) // Output Value Register
#define PIO_OVRS_OFFS                (0x54) // Output Value Register Set
#define PIO_OVRC_OFFS                (0x58) // Output Value Register Clear
#define PIO_OVRT_OFFS                (0x5C) // Output Value Register Toogle

/****** Target specific configuration *******************************/
#define LEDPORT_BASE_ADDR            (AVR32_GPIO_PIOB_BASE_ADDR)
#define LED1_BIT                     (27)   // LED1 
#define LED2_BIT                     (28)   // LED2 
#define LED3_BIT                     (29)   // LED3 
#define LED4_BIT                     (30)   // LED4 
#define LED5GREEN_BIT                (19)   // LED5/Green 
#define LED5RED_BIT                  (20)   // LED5/Red 
#define LED6GREEN_BIT                (21)   // LED6/Green
#define LED6RED_BIT                  (22)   // LED6/Red

#define LEDPORT_GPER                 (*(volatile unsigned int*) (LEDPORT_BASE_ADDR + PIO_GPER_OFFS))
#define LEDPORT_ODR                  (*(volatile unsigned int*) (LEDPORT_BASE_ADDR + PIO_ODR_OFFS))
#define LEDPORT_OVRS                 (*(volatile unsigned int*) (LEDPORT_BASE_ADDR + PIO_OVRS_OFFS))
#define LEDPORT_OVRC                 (*(volatile unsigned int*) (LEDPORT_BASE_ADDR + PIO_OVRC_OFFS))
#define LEDPORT_OVRT                 (*(volatile unsigned int*) (LEDPORT_BASE_ADDR + PIO_OVRT_OFFS))
#define LEDPORT_STATE                (*(volatile unsigned int*) (LEDPORT_BASE_ADDR + PIO_OVR_OFFS))

#define LEDMASK_ALL                   ((1uL << LED1_BIT)      \
                                     | (1uL << LED2_BIT)      \
                                     | (1uL << LED3_BIT)      \
                                     | (1uL << LED4_BIT)      \
                                     | (1uL << LED5GREEN_BIT) \
                                     | (1uL << LED5RED_BIT)   \
                                     | (1uL << LED6GREEN_BIT) \
                                     | (1uL << LED6RED_BIT))

/*********************************************************************
*
*       Global functions
*
**********************************************************************
*/

/*********************************************************************
*
*       BSP_Init()
*/
void BSP_Init(void) {
  LEDPORT_GPER  = LEDMASK_ALL;  // Enable Port           
  LEDPORT_ODR   = LEDMASK_ALL;  // Switch to output mode 
  LEDPORT_OVRS |= LEDMASK_ALL;  // Initially clear LEDs  
}

/*********************************************************************
*
*       LED switching routines
*       LEDs are switched on with low level on port lines
*/
void BSP_SetLED(int Index) {
  if (Index < 4) { 
    LEDPORT_OVRC = (1uL << (Index + LED1_BIT));             // Switch on LED  
  } else if (Index < 8) {                            
    LEDPORT_OVRC = (1uL << ((Index - 4) + LED5GREEN_BIT));  // Switch on LED      
  }
}

void BSP_ClrLED(int Index) {
  if (Index < 4) { 
    LEDPORT_OVRS = (1uL << (Index + LED1_BIT));             // Switch off LED  
  } else if (Index < 8) {                            
    LEDPORT_OVRS = (1uL << ((Index - 4) + LED5GREEN_BIT));  // Switch off LED      
  }
}

void BSP_ToggleLED(int Index) {
  if (Index < 4) { 
    LEDPORT_OVRT = (1uL << (Index + LED1_BIT));             // Toogle LED  
  } else if (Index < 8) {                            
    LEDPORT_OVRT = (1uL << ((Index - 4) + LED5GREEN_BIT));  // Toogle LED      
  }
}

/********************* End of File **********************************/
