/*********************************************************************
*               SEGGER MICROCONTROLLER GmbH & Co KG                  *
*       Solutions for real time microcontroller applications         *
**********************************************************************
*                                                                    *
*       (c) 1995 - 2011  SEGGER Microcontroller GmbH & Co KG         *
*                                                                    *
*       www.segger.com     Support: support@segger.com               *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS * Real time operating system for microcontrollers      *
*                                                                    *
*                                                                    *
*       Please note:                                                 *
*                                                                    *
*       Knowledge of this file may under no circumstances            *
*       be used to write a similar product or a real-time            *
*       operating system for in-house use.                           *
*                                                                    *
*       Thank you for your fairness !                                *
*                                                                    *
**********************************************************************
*                                                                    *
*       OS version: 3.82r                                            *
*                                                                    *
**********************************************************************

----------------------------------------------------------------------
File    : RTOSInit_AT32UC3A.c 
Purpose : Initializes and handles the hardware for the OS as far
          as required by the OS for Atmel AT32UC3A0 CPU.
--------  END-OF-HEADER  ---------------------------------------------
*/

#include "RTOS.h"
#include <intrinsics.h>

/*********************************************************************
*
*       Configuration
*
*********************************************************************/

/*********************************************************************
*
*       Clock frequency settings
*/
#define   OS_FSYS       (48000000uL)

#ifndef   OS_TICK_FREQ
  #define OS_TICK_FREQ  (1000)
#endif

#ifndef   OS_PCLK_TIMER                /* Clock used for embOS timer */
  #define OS_PCLK_TIMER (OS_FSYS)
#endif

#ifndef   OS_PCLK_UART                 /* Peripheral clock for UART  */
  #define OS_PCLK_UART  (OS_FSYS)
#endif

/*********************************************************************
*
*       UART settings for OSView
*       If you do not want (or can not due to hardware limitations)
*       to dedicate a UART to OSView, please define it to be -1
*       Currently the standard code enables UART 0 per default
*/
#ifndef   OS_VIEW_ENABLE
  #define OS_VIEW_ENABLE  (1)
#endif

#ifndef   OS_UART
  #define OS_UART (0)
#endif

#ifndef   OS_BAUDRATE
  #define OS_BAUDRATE (38400uL)
#endif

/*********************************************************************
*
*       UART baudrate calculation
*
*********************************************************************/

#define BAUDRATEREG ((OS_PCLK_UART + OS_BAUDRATE*8L) / (OS_BAUDRATE*16))
#if (BAUDRATEREG > 0xFFFFuL)
  #error Cannot set up UART for the given OS_FSYS and OS_BAUDRATE values.
#endif

/****** End of configuration settings *******************************/

#define OS_UART_USED (OS_VIEW_ENABLE && ((OS_UART == 0) || (OS_UART == 1)))

#define OS_TIMER_RELOAD (OS_PCLK_TIMER/OS_TICK_FREQ)

/*********************************************************************
*
*       Local defines (sfrs used in RTOSInit.c)
*
**********************************************************************
*/
#define COUNT                 (0x108uL)
#define COMPARE               (0x10cuL)

#define MCCTRL                (*(volatile unsigned int*) (0xFFFF0C00uL))
#define CKSEL                 (*(volatile unsigned int*) (0xFFFF0C04uL))
#define PLL0                  (*(volatile unsigned int*) (0xFFFF0C20uL))
#define OSCCTRL0              (*(volatile unsigned int*) (0xFFFF0C28uL))
#define ISR                   (*(volatile unsigned int*) (0xFFFF0C4CuL))
#define GCCTRL                (*(volatile unsigned int*) (0xFFFF0C60uL))
#define FCR                   (*(volatile unsigned int*) (0xFFFE1400uL))
#define IRR0                  (*(volatile unsigned int*) (0xFFFF0900uL))

#define GPIO_BASE_ADDR        (0xFFFF1000)
#define GPIO_PORT0_BASE_ADDR  (GPIO_BASE_ADDR + 0x0000)
#define GPIO_PORT1_BASE_ADDR  (GPIO_BASE_ADDR + 0x0100)
#define GPIO_PORT2_BASE_ADDR  (GPIO_BASE_ADDR + 0x0200)
#define GPIO_PORT3_BASE_ADDR  (GPIO_BASE_ADDR + 0x0300)
#define GPIO_PORT4_BASE_ADDR  (GPIO_BASE_ADDR + 0x0400)

#define GPIO_PORT1_GPERC      (*(volatile unsigned int*) (GPIO_PORT1_BASE_ADDR + 0x08))
#define GPIO_PORT1_PMR0S      (*(volatile unsigned int*) (GPIO_PORT1_BASE_ADDR + 0x14))
#define GPIO_PORT1_PMR0C      (*(volatile unsigned int*) (GPIO_PORT1_BASE_ADDR + 0x18))
#define GPIO_PORT1_PMR1S      (*(volatile unsigned int*) (GPIO_PORT1_BASE_ADDR + 0x24))
#define GPIO_PORT1_PMR1C      (*(volatile unsigned int*) (GPIO_PORT1_BASE_ADDR + 0x28))
#define GPIO_PORT2_GPERC      (*(volatile unsigned int*) (GPIO_PORT2_BASE_ADDR + 0x08))
#define GPIO_PORT2_PMR0S      (*(volatile unsigned int*) (GPIO_PORT2_BASE_ADDR + 0x14))
#define GPIO_PORT2_PMR0C      (*(volatile unsigned int*) (GPIO_PORT2_BASE_ADDR + 0x18))
#define GPIO_PORT2_PMR1S      (*(volatile unsigned int*) (GPIO_PORT2_BASE_ADDR + 0x24))
#define GPIO_PORT2_PMR1C      (*(volatile unsigned int*) (GPIO_PORT2_BASE_ADDR + 0x28))
#define GPIO_PORT3_GPERC      (*(volatile unsigned int*) (GPIO_PORT3_BASE_ADDR + 0x08))
#define GPIO_PORT3_PMR0S      (*(volatile unsigned int*) (GPIO_PORT3_BASE_ADDR + 0x14))
#define GPIO_PORT3_PMR0C      (*(volatile unsigned int*) (GPIO_PORT3_BASE_ADDR + 0x18))
#define GPIO_PORT3_PMR1S      (*(volatile unsigned int*) (GPIO_PORT3_BASE_ADDR + 0x24))
#define GPIO_PORT3_PMR1C      (*(volatile unsigned int*) (GPIO_PORT3_BASE_ADDR + 0x28))

#define HMATRIX_BASE_ADDR     (0xFFFE1000)
#define HMATRIX_SFR5          (*(volatile unsigned int*) (HMATRIX_BASE_ADDR + 0x124))

#define SDRAMC_BASE_ADDR      (0xFFFE2000)
#define SDRAMC_MR             (*(volatile unsigned int*) (SDRAMC_BASE_ADDR + 0x00))   // Mode Register
#define SDRAMC_TR             (*(volatile unsigned int*) (SDRAMC_BASE_ADDR + 0x04))   // Refresh Timer Register
#define SDRAMC_CR             (*(volatile unsigned int*) (SDRAMC_BASE_ADDR + 0x08))   // Configuration Register
#define AVR32_EBI_SDRAM_CS    1

#if OS_UART_USED

#define USART0_BASE_ADDR      (0xFFFF1400uL)
#define USART1_BASE_ADDR      (0xFFFF1800uL)

#if   (OS_UART == 0)
  #define USART_BASE_ADDR     USART0_BASE_ADDR
  #define OS_UART_PIN_MASK    (0x3uL << 0)
  #define USART_FUNC_A        (1)
  #define USART_FUNC_B        (0)
  #define USART_FUNC_C        (0)
#elif (OS_UART == 1)
  #define USART_BASE_ADDR     USART1_BASE_ADDR
  #define OS_UART_PIN_MASK    (0x3uL << 5)
  #define USART_FUNC_A        (1)
  #define USART_FUNC_B        (0)
  #define USART_FUNC_C        (0)
#endif

#define USART_CR              (*(volatile unsigned int*) (USART_BASE_ADDR + 0x00))
#define USART_MR              (*(volatile unsigned int*) (USART_BASE_ADDR + 0x04))
#define USART_IER             (*(volatile unsigned int*) (USART_BASE_ADDR + 0x08))
#define USART_IDR             (*(volatile unsigned int*) (USART_BASE_ADDR + 0x0C))
#define USART_IMR             (*(volatile unsigned int*) (USART_BASE_ADDR + 0x10))
#define USART_CSR             (*(volatile unsigned int*) (USART_BASE_ADDR + 0x14))
#define USART_RHR             (*(volatile unsigned int*) (USART_BASE_ADDR + 0x18))
#define USART_THR             (*(volatile unsigned int*) (USART_BASE_ADDR + 0x1C))
#define USART_BRGR            (*(volatile unsigned int*) (USART_BASE_ADDR + 0x20))
#define USART_RTOR            (*(volatile unsigned int*) (USART_BASE_ADDR + 0x24))
#define USART_TTGR            (*(volatile unsigned int*) (USART_BASE_ADDR + 0x28))

#define PORT0_BASE_ADDR       (0xFFFF1000uL)
#define PORT0_GPERS           (*(volatile unsigned int*) (PORT0_BASE_ADDR + 0x04))
#define PORT0_GPERC           (*(volatile unsigned int*) (PORT0_BASE_ADDR + 0x08))
#define PORT0_PMR0S           (*(volatile unsigned int*) (PORT0_BASE_ADDR + 0x14))
#define PORT0_PMR0C           (*(volatile unsigned int*) (PORT0_BASE_ADDR + 0x18))
#define PORT0_PMR1S           (*(volatile unsigned int*) (PORT0_BASE_ADDR + 0x24))
#define PORT0_PMR1C           (*(volatile unsigned int*) (PORT0_BASE_ADDR + 0x28))
#define PORT0_ODERC           (*(volatile unsigned int*) (PORT0_BASE_ADDR + 0x48))

#define TXE_BIT                (1)

#endif  /* OS_UART_USED */

/*********************************************************************
*
*       Static data
*
**********************************************************************
*/

/*********************************************************************
*
*       Local functions
*
**********************************************************************
*/

/*********************************************************************
*
*       _OS_ISR_TickHandler()
*
*       The embOS timer tick handler function, called from _OS_ISR_Tick()
*/
static void _OS_ISR_TickHandler(void) {
  __set_system_register(COMPARE, OS_TIMER_RELOAD);
  OS_TICK_Handle();
}

/*********************************************************************
*
*       _OS_ISR_Tick()
*
*       The embOS timer tick handler, called from compare interrupt
*/
#pragma handler = 0, 0
static __interrupt void _OS_ISR_Tick(void) {
  OS_CallISR(_OS_ISR_TickHandler);
}

/*********************************************************************
*
*       SDRAMC_ClkDelay()
*
*       Makes a delay for SDRAM initialization
*/
static void SDRAMC_ClkDelay(unsigned long ck) {
  //
  // Use the CPU cycle counter (CPU and HSB clocks are the same).
  //
  unsigned long delay_start_cycle = __get_system_register(COUNT);
  unsigned long delay_end_cycle = delay_start_cycle + ck;
  //
  // To be safer, the end of wait is based on an inequality test, so CPU cycle
  // counter wrap around is checked.
  //
  if (delay_start_cycle > delay_end_cycle) {
    while ((unsigned long)__get_system_register(COUNT) > delay_end_cycle);
  } while ((unsigned long)__get_system_register(COUNT) < delay_end_cycle);
}

/*********************************************************************
*
*       SDRAM_Init()
*
*       Initialize the external SDRAM
*/
static void SDRAM_Init(void) {
  //
  // Initialize the external SDRAM chip
  //
  unsigned long hsb_hz     = OS_FSYS;
  unsigned long hsb_mhz_dn = hsb_hz / 1000000;
  unsigned long hsb_mhz_up = (hsb_hz + 999999) / 1000000;
  volatile OS_U16 *sdram   = ((void *)0xD0000000);
  unsigned int i;

  //
  // Put the multiplexed MCU pins used for the SDRAM under control of the SDRAMC.
  //
  GPIO_PORT2_PMR0C = (1uL <<  6) | (1uL <<  8) | (1uL <<  9) | (1uL << 10) | (1uL << 11) | (1uL << 12) | (1uL << 13) | (1uL << 14) | (1uL << 15) | (1uL << 16) | (1uL << 17) | (1uL << 19) | (1uL << 20) | (1uL << 22) | (1uL << 23) | (1uL << 26) | (1uL << 27) | (1uL << 28) | (1uL << 29) | (1uL << 30) | (1uL << 31);
  GPIO_PORT2_PMR1C = (1uL <<  6) | (1uL <<  8) | (1uL <<  9) | (1uL << 10) | (1uL << 11) | (1uL << 12) | (1uL << 13) | (1uL << 14) | (1uL << 15) | (1uL << 16) | (1uL << 17) | (1uL << 19) | (1uL << 20) | (1uL << 22) | (1uL << 23) | (1uL << 26) | (1uL << 27) | (1uL << 28) | (1uL << 29) | (1uL << 30) | (1uL << 31);
  GPIO_PORT2_GPERC = (1uL <<  6) | (1uL <<  8) | (1uL <<  9) | (1uL << 10) | (1uL << 11) | (1uL << 12) | (1uL << 13) | (1uL << 14) | (1uL << 15) | (1uL << 16) | (1uL << 17) | (1uL << 19) | (1uL << 20) | (1uL << 22) | (1uL << 23) | (1uL << 26) | (1uL << 27) | (1uL << 28) | (1uL << 29) | (1uL << 30) | (1uL << 31);

  GPIO_PORT3_PMR0C = (1uL <<  0) | (1uL <<  1) | (1uL <<  2) | (1uL <<  3) | (1uL <<  4) | (1uL <<  5) | (1uL <<  6) | (1uL <<  7) | (1uL <<  8) | (1uL <<  9) | (1uL << 10) | (1uL <<  13);
  GPIO_PORT3_PMR1C = (1uL <<  0) | (1uL <<  1) | (1uL <<  2) | (1uL <<  3) | (1uL <<  4) | (1uL <<  5) | (1uL <<  6) | (1uL <<  7) | (1uL <<  8) | (1uL <<  9) | (1uL << 10) | (1uL <<  13);
  GPIO_PORT3_GPERC = (1uL <<  0) | (1uL <<  1) | (1uL <<  2) | (1uL <<  3) | (1uL <<  4) | (1uL <<  5) | (1uL <<  6) | (1uL <<  7) | (1uL <<  8) | (1uL <<  9) | (1uL << 10) | (1uL <<  13);

  GPIO_PORT1_PMR0C = (1uL << 10) | (1uL << 11) | (1uL << 12) | (1uL << 13) | (1uL << 14) | (1uL << 16);
  GPIO_PORT1_PMR1S = (1uL << 10) | (1uL << 11) | (1uL << 12) | (1uL << 13) | (1uL << 14) | (1uL << 16);
  GPIO_PORT1_GPERC = (1uL << 10) | (1uL << 11) | (1uL << 12) | (1uL << 13) | (1uL << 14) | (1uL << 16);

  HMATRIX_SFR5 |= (1uL << AVR32_EBI_SDRAM_CS); // Enable SDRAM mode for CS1
  HMATRIX_SFR5;

  //
  // Configure the SDRAM Controller with SDRAM setup and timing information.
  // All timings below are rounded up because they are minimal values.
  //
  SDRAMC_CR =
      ((( 9 - 8)                         <<  0)   & 0x00000003) |
      ((( 13 - 11)                       <<  2)   & 0x0000000c) |
      ((( 2 - 1)                         <<  4)   & 0x00000010) |
      ((  2                              <<  5)   & 0x00000060) |
      ((( 16 >> 4)                       <<  7)   & 0x00000080) |
      ((((14 * hsb_mhz_up + 999) / 1000) <<  8)   & 0x00000f00) |
      ((((60 * hsb_mhz_up + 999) / 1000) << 12)   & 0x0000f000) |
      ((((15 * hsb_mhz_up + 999) / 1000) << 16)   & 0x000f0000) |
      ((((15 * hsb_mhz_up + 999) / 1000) << 20)   & 0x00f00000) |
      ((((37 * hsb_mhz_up + 999) / 1000) << 24)   & 0x0f000000) |
      ((((67 * hsb_mhz_up + 999) / 1000) << 28)   & 0xf0000000);
  SDRAMC_CR;
  //
  // Issue a NOP command to the SDRAM in order to start the generation of SDRAMC signals.
  //
  SDRAMC_MR = 0x00000001;
  SDRAMC_MR;
  sdram[0];
  //
  // Wait during the SDRAM stable-clock initialization delay.
  //
  SDRAMC_ClkDelay((100) * (hsb_mhz_up));
  //
  // Issue a PRECHARGE ALL command to the SDRAM.
  //
  SDRAMC_MR = 0x00000002;
  SDRAMC_MR;
  sdram[0];
  SDRAMC_ClkDelay(((15) * (hsb_mhz_up) + 999) / 1000);

  // Issue initialization AUTO REFRESH commands to the SDRAM.
  SDRAMC_MR = 0x00000004;
  SDRAMC_MR;
  for (i = 0; i < 2; i++) {
    sdram[0];
    SDRAMC_ClkDelay(((66) * (hsb_mhz_up) + 999) / 1000);
  }

  // Issue a LOAD MODE REGISTER command to the SDRAM.
  // This configures the SDRAM with the following parameters in the mode register:
  //  - bits 0 to 2: burst length: 1 (000b);
  //  - bit 3: burst type: sequential (0b);
  //  - bits 4 to 6: CAS latency: AVR32_SDRAMC.CR.cas;
  //  - bits 7 to 8: operating mode: standard operation (00b);
  //  - bit 9: write burst mode: programmed burst length (0b);
  //  - all other bits: reserved: 0b.
  SDRAMC_MR = 0x00000003;
  SDRAMC_MR;
  sdram[0];
  SDRAMC_ClkDelay(((2) * (hsb_mhz_up) + 999) / 1000);
  //
  // Switch the SDRAM Controller to normal mode.
  //
  SDRAMC_MR = 0x00000000;
  SDRAMC_MR;
  sdram[0];
  //
  // Write the refresh period into the SDRAMC Refresh Timer Register.
  // tR is rounded down because it is a maximal value.
  //
  SDRAMC_TR = (7812 * hsb_mhz_dn) / 1000;
  SDRAMC_TR;
}

/*********************************************************************
*
*       Global functions
*
**********************************************************************
*/

/*********************************************************************
*
*       OS_InitHW()
*
*       Initialize the hardware (timer) required for the OS to run.
*       May be modified, if an other timer should be used
*/
void OS_InitHW(void) {
  OS_IncDI();
  //
  // Initialize PLL
  //
  OSCCTRL0    = 0x00000707uL;           // Mode 1 and 560 usec startup time
  MCCTRL      = 0x00000004uL;           // Enable Oscillator 0
  while ((ISR & 0x00000080uL) == 0);    // Wait until clock ready
  MCCTRL      = 0x00000005uL;           // Select Oscillator 0 as source for the main clock
  PLL0        = 0x100F020CuL;           // Mul = 16, div=2, pllcount=16, OPT = 3 => fvco = 96MHz with 80MHz <fvco <180MHz, fPLL = fvco /2
  PLL0        = 0x100F020DuL;           // Enable PLL0
  while ((ISR & 0x00000001uL) == 0);    // Wait until ready to be used as clock source
  GCCTRL      = 0x00000002uL;           // Select PLL 0 as source for the generic clock, no division
  GCCTRL      = 0x00000006uL;           // Enable Generic Clock 0
  CKSEL       = 0x00000000uL;           // PBADIV = 0: PBA clock equals main clock
  while ((ISR & 0x00000020uL) == 0);    // Wait until ready to be used as clock source
  FCR         = 0x00000040uL;           // Set 1 wait state for Flash
  MCCTRL      = 0x00000006uL;           // Select PLL0 as source for the main clock
  //
  // Initialize SDRAM
  //
  SDRAM_Init();
  //
  // Initialize OS timer
  //
  __set_system_register(COMPARE, OS_TIMER_RELOAD);
  __set_system_register(COUNT, 0);
  OS_COM_Init();
  OS_DecRI();
}

/*********************************************************************
*
*       Idle loop  OS_Idle()
*
*       Please note:
*       This is basically the "core" of the idle loop.
*       This core loop can be changed, but:
*       The idle loop does not have a stack of its own, therefore no
*       functionality should be implemented that relies on the stack
*       to be preserved. However, a simple program loop can be programmed
*       (like toggling an output or incrementing a counter)
*/
void OS_Idle(void) {     // Idle loop: No task is ready to exec
  while (1) {
#if DEBUG == 0
// Instruction to put CPU in halt mode to save power
#endif
  }
}

/*********************************************************************
*
*       OS_GetTime_Cycles()
*
*       This routine is required for high
*       resolution time maesurement functions.
*       It returns the system time in timer clock cycles.
*/
OS_U32 OS_GetTime_Cycles(void) {
  OS_U32 Time;
  int    Count;

  Time  = OS_Time;
  Count = OS_GetSVCCount();
  if (IRR0 & 0x01) {  // Counter interrupt pending, adjust time
    Time++;
    Count = OS_GetSVCCount();
  }
  return (OS_TIMER_RELOAD) * Time + Count;
}

/*********************************************************************
*
*       OS_ConvertCycles2us()
*
*       Convert Cycles into micro seconds.
*
*       If your clock frequency is not a multiple of 1 MHz,
*       you may have to modify this routine in order to get proper
*       diagonstics.
*
*       This routine is required for profiling or high resolution time
*       measurement only. It does not affect operation of the OS.
*/
OS_U32 OS_ConvertCycles2us(OS_U32 Cycles) {
  return Cycles/(OS_PCLK_TIMER/1000000);
}

/*********************************************************************
*
*       Communication for OSView via UART (optional)
*
**********************************************************************
*/

#if OS_UART_USED

/*********************************************************************
*
*       _UartHandler()
*
*       Used for embOSView, called from OS_COM_ISR_Usart().
*/
static void _UartHandler(void) {
  char c;
  if ((USART_CSR & (1uL << 0)) != 0) {        // Character received?
    c = USART_RHR;                            // Process character, ignore errors
    OS_OnRx(c);
  }
  if (((USART_CSR & (1uL << TXE_BIT)) != 0) && ((USART_IMR & (1uL << TXE_BIT)) != 0)) {
    USART_IDR = (1uL << TXE_BIT);             // Disable Tx interrupt
    OS_OnTx();                                // Send next, if any...
  }
  USART_CR = (1uL << 8);                      // Reset any errors
}

/*********************************************************************
*
*       OS_COM_ISR_Usart()
*       OS USART interrupt handler
*       handles both, Rx and Tx interrupt
*/
#if   (OS_UART == 0)
  #pragma handler = 5, 0
#elif (OS_UART == 1)
  #pragma handler = 6, 0
#endif
static __interrupt void OS_COM_ISR_Usart(void) {
  OS_CallISR(_UartHandler);
}

/*********************************************************************
*
*       OS_COM_Send1()
*       Never call this function directly from your application
*/
void OS_COM_Send1(OS_U8 c) {
  USART_THR = c;                  // Send character
  USART_IER = (1uL << TXE_BIT);   // Enable interrupt for TxRDY / TxEmpty
}

/*********************************************************************
*
*       OS_COM_Init()
*       Initialize UART for OSView
*/
void OS_COM_Init(void) {            /* Initialize UART, enable UART interrupts */
  PORT0_ODERC = OS_UART_PIN_MASK;   // Disable output driver of UART pins
  PORT0_GPERS = OS_UART_PIN_MASK;   // Temporarily set UART pins to PIO mode
  PORT0_PMR0C = OS_UART_PIN_MASK;   // Clear Configuration bit0
  PORT0_PMR1C = OS_UART_PIN_MASK;   // Clear Configuration bit1, select Function A
  PORT0_PMR0S = (OS_UART_PIN_MASK & USART_FUNC_B);
  PORT0_PMR1S = (OS_UART_PIN_MASK & USART_FUNC_C);
  PORT0_GPERC = OS_UART_PIN_MASK;   // Enable special function on UART pins


  USART_IDR   = 0xFFFFFFFFuL;       // Disable all uart interrupts
  USART_MR    = 0x00000000uL;       // Reset mode register
  USART_RTOR  = 0x00000000uL;       // Reset receive timeout register
  USART_TTGR  = 0x00000000uL;       // Reset transmitter timeguard register
  USART_CR    = 0x000A610CuL;       // Reset usart, disable dtr and rts
  USART_BRGR  = BAUDRATEREG;        // Set baudrate to selected baud rate
  USART_MR    = 0x000008C0Ul;       // Set mode: Oversampling mode 16, Normal, MCK, 8 data bits, async mode, NO parity, 1 stop bit
  USART_IER   = 0x00000001uL;       // Interrupt enabled for: RXRDY
  USART_CR   |= (1uL << 8);         // Reset any Errors
  USART_CR   |= 0x00000050uL;       // Enable rx and tx
}

#else  /* selected UART not supported, using dummies */

void OS_COM_Init(void) {}
void OS_COM_Send1(OS_U8 c) {
  OS_USEPARA(c);           /* Avoid compiler warning */
  OS_COM_ClearTxActive();  /* Let the OS know that Tx is not busy */
}

#endif /*  OS_UART_USED  */

/****** Final check of configuration ********************************/

#ifndef OS_UART_USED
  #error "OS_UART_USED has to be defined"
#endif

/*****  End Of File  ************************************************/
