/******************************************************************************
 *
 * Segment initialization that must be performed before main is called.
 *
 * Used with ICCAVR32.
 *
 * Copyright 1998,2003,2005 IAR Systems. All rights reserved.
 *
 * $Revision: 2.1 $
 *
 *****************************************************************************/

#include "segment_init.h"

/* We're using extended keywords in this file. Make sure */
/* that the compiler has IAR extensions enabled.         */
#pragma language=extended

/* Declare the segment initialization table segment. */
#pragma segment="INITTAB"

/* If you are using C or EC++ code then you must include  */
/* the segment initialization code below or supply your   */
/* own version of segment initialization code. Otherwise  */
/* we cannot guarantee that any C or EC++ code will work. */
/* If you're only using assembler modules then you don't  */
/* need to initialize any segments at all.                */

#ifdef __cplusplus
extern "C" {
#endif

void __segment_init(void)
{
  /* Initialize the INITTAB pointer. */
  __SegmentInitBlock_Type __data32 * table    = __segment_begin("INITTAB");
  __SegmentInitBlock_Type __data32 * tableEnd = __segment_end  ("INITTAB");

  /* Loop over all copy/zero blocks in the INITTAB segment */
  do
  {
    /* Fetch initialization data from the INITTAB segment. */
    unsigned char __data32 * src  = table->mSource;
    unsigned char __data32 * dst  = table->mDestination;
    unsigned long            size = table->mCount;

    ++table;

    /* If the src is 0 then we should clear a memory */
    /* block, otherwise it's a copy operation.       */
    if (src != dst)
    {
      do { *dst++ = *src++; } while(--size != 0);
    }
    else
    {
      do { *dst++ = 0; } while(--size != 0);
    }
  }  while(table < tableEnd);
}

#ifdef __cplusplus
}
#endif
