/******************************************************************************
 *
 * Data structure required for segment initialization.
 *
 * The ICCAVR32 compiler will automatically generate a segment called INITTAB,
 * containing initialization information for compiler generated data segments.
 * The INITTAB segment is an array of __SegmentInitBlock_Type structures.
 *
 * Copyright 1998,2003,2005 IAR Systems. All rights reserved.
 *
 * $Revision: 2.1 $
 *
 *****************************************************************************/

#ifndef __SEGMENT_INIT_H
#define __SEGMENT_INIT_H

#pragma language=extended

/* A __SegmentInitBlock_Type record describes one block  */
/* that should be initialized during system startup. The */
/* INITTAB segment contains an unspecified number of     */
/* __SegmentInitBlock_Type records.                      */
typedef struct
{
  /* If mSource is equal to mDestination then the block */
  /* pointed to by mDestination should be cleared. If   */
  /* mSOurce is not equal to mDestination then mSource  */
  /* points to data that should be copied to the block  */
  /* pointed to by mDestination.                        */
  unsigned char __data32 * mSource;

  /* mDestination points to the block that should */
  /* be initialized, either by setting all bytes  */
  /* to 0 or copying data from the block pointed  */
  /* to by mSource.                               */
  unsigned char __data32 * mDestination;

  /* mCount determines the size of the block */
  /* pointed to by mDestination.             */
  unsigned long mCount;
} __SegmentInitBlock_Type;

#pragma language=default

#ifdef __cplusplus
extern "C" {
#endif

void __segment_init(void);

#ifdef __cplusplus
}
#endif


#endif /* __SEGMENT_INIT_H */
